@@ -1,3 +1,4 @@
+from com import msgs
 from storage import Device, File
 from utils import mimetypes
 from utils import maemo
@@ -46,7 +47,7 @@ class LocalDevice(Device):
 
     def get_icon(self):
     
-        return theme.mb_device_n800
+        return theme.mb_device_nit
 
 
     def get_root(self):
@@ -54,12 +55,12 @@ class LocalDevice(Device):
         f = File(self)
         f.is_local = True
         f.path = "MENU"
-        f.mimetype = f.DIRECTORY
+        f.mimetype = f.DEVICE_ROOT
         f.resource = ""
         f.name = self.__name
         f.info = "Browse the filesystem"
-        f.folder_flags = f.ITEMS_ENQUEUEABLE | \
-                         f.INDEXABLE
+        f.icon = self.get_icon().get_path()
+        f.folder_flags = f.ITEMS_ENQUEUEABLE
 
         return f
         
@@ -114,7 +115,26 @@ class LocalDevice(Device):
         #end for
         
         cb(None, *args)        
+
+
+    def __on_add_to_playlist(self, folder, f):
+    
+        self.emit_message(msgs.PLAYLIST_ACT_APPEND, "", f)
+
+
+    def __on_put_on_dashboard(self, folder, f):
         
+        f.bookmarked = True
+
+
+    def get_file_actions(self, folder, f):
+    
+        actions = []
+        actions.append((None, "Add to Playlist", self.__on_add_to_playlist))
+        actions.append((None, "Put on Dashboard", self.__on_put_on_dashboard))
+        
+        return actions
+
         
     def get_file(self, path):
 
