@@ -26,10 +26,6 @@ class AudioAlbumStorage(AudioArtistStorage):
         AudioArtistStorage.__init__(self)
 
 
-    def handle_message(self, msg, *args):
-    
-        pass
-
         
     def get_prefix(self):
     
@@ -57,7 +53,7 @@ class AudioAlbumStorage(AudioArtistStorage):
         f.resource = ""
         f.name = self.get_name()
         f.info = "Browse your music library by album"
-        f.folder_flags = f.ITEMS_ENQUEUEABLE
+        f.folder_flags = f.ITEMS_ENQUEUEABLE | f.ITEMS_COMPACT
         
         return f
           
@@ -65,7 +61,11 @@ class AudioAlbumStorage(AudioArtistStorage):
     
     def ls_async(self, path, cb, *args):
 
-        self._check_for_updated_media()
+        try:
+            self._check_for_updated_media()
+        except:
+            from utils import logging
+            print logging.stacktrace()
 
         if (not path.endswith("/")): path += "/"
         parts = [ p for p in path.split("/") if p ]
