@@ -32,13 +32,12 @@ class AudioArtistStorage(Device):
 
         
  
-    def handle_message(self, msg, *args):
+    def handle_MEDIASCANNER_EV_SCANNING_FINISHED(self):
     
-        if (msg == msgs.MEDIASCANNER_EV_SCANNING_FINISHED):
-            #self.__index.schedule_scanner(self.__update_media)
-            #self.__update_media()
-            #self.__index.save()
-            self._media_was_updated[0] = True
+        #self.__index.schedule_scanner(self.__update_media)
+        #self.__update_media()
+        #self.__index.save()
+        self._media_was_updated[0] = True
 
 
     def __update_media(self):
@@ -50,11 +49,11 @@ class AudioArtistStorage(Device):
             for folder in added:
                 cnt += 1
                 percent = int((cnt / float(total_length)) * 100)
-                self.emit_event(msgs.UI_ACT_SHOW_MESSAGE,
-                                "Updating Index",
-                                #"- %s -" % folder.name,
-                                "- %d%% complete -" % (percent),
-                                self.get_icon())
+                self.emit_message(msgs.UI_ACT_SHOW_MESSAGE,
+                                  "Updating Index",
+                                  #"- %s -" % folder.name,
+                                  "- %d%% complete -" % (percent),
+                                  self.get_icon())
                                 
                 self.__index.remove_album(folder)
                 self.__index.add_album(folder)
@@ -65,7 +64,7 @@ class AudioArtistStorage(Device):
                 self.__index.remove_album(folder)
             #end for
 
-            self.emit_event(msgs.UI_ACT_HIDE_MESSAGE)
+            self.emit_message(msgs.UI_ACT_HIDE_MESSAGE)
             #finished.set()
           
 
@@ -135,10 +134,12 @@ class AudioArtistStorage(Device):
         f.is_local = True
         f.can_skip = True
         f.path = "/"
-        f.mimetype = f.DIRECTORY
+        f.mimetype = f.DEVICE_ROOT
         f.resource = ""
         f.name = self.get_name()
+        f.icon = self.get_icon().get_path()
         f.info = "Browse your music library by artist"
+        f.folder_flags = f.ITEMS_COMPACT
         
         return f
           
