@@ -24,11 +24,10 @@ class AudioStorage(Device):
         Device.__init__(self)
         
         
-    def handle_message(self, msg, *args):
+    def handle_MEDIASCANNER_EV_SCANNING_FINISHED(self):
     
-        if (msg == msgs.MEDIASCANNER_EV_SCANNING_FINISHED):
-            #self.__update_media()
-            self.__media_was_updated = True
+        #self.__update_media()
+        self.__media_was_updated = True
 
 
     def __update_media(self):
@@ -66,9 +65,10 @@ class AudioStorage(Device):
         f.is_local = True
         f.path = "/"
         f.can_skip = True
-        f.mimetype = f.DIRECTORY
+        f.mimetype = f.DEVICE_ROOT
         f.resource = ""
         f.name = self.get_name()
+        f.icon = self.get_icon().get_path()
         f.info = "Browse your music library by folders"
         
         return f
