@@ -1,19 +1,19 @@
-from com import Component, Viewer, Widget, msgs
-import components
+from com import Component, Viewer, View, Widget, msgs
 from utils import logging
 
 from RootPane import RootPane
 from TitlePanel import TitlePanel
-from ControlPanel import ControlPanel
 from ViewerState import ViewerState
+from ui.layout import VBox
 from ui.Image import Image
 from ui.Window import Window
 from ui.Button import Button
 from ui.Pixmap import Pixmap
+from ui.Tabs import Tabs
 from ui.Widget import Widget as UIWidget
 from mediabox import config
 from mediabox import values
-from utils import maemo
+import platforms
 from mediabox import viewmodes
 from theme import theme
 
@@ -34,10 +34,14 @@ class AppWindow(Component, RootPane):
 
     def __init__(self):
 
-        self.__viewers = []
+        self.__views = []
+        self.__current_view = None
+        self.__initial_view = None
+
+        #self.__viewers = []
         self.__current_viewer = None
         self.__current_device_id = ""
-        self.__view_mode = viewmodes.TITLE_ONLY
+        self.__view_mode = viewmodes.NORMAL
         self.__battery_remaining = 100.0
         
         # auto repeat handler for hw keys without auto repeat
@@ -58,71 +62,59 @@ class AppWindow(Component, RootPane):
         # flag for indicating whether initialization has finished
         self.__is_initialized = False
     
-        if (maemo.IS_MAEMO):
+        if (platforms.PLATFORM == platforms.MAEMO4):
+            platforms.create_osso_context(values.OSSO_NAME, "1.0", False)
             import hildon
-            import osso
-            maemo.set_osso_context(osso.Context(values.OSSO_NAME,
-                                                "1.0", False))
             self.__program = hildon.Program()
-    
+            
+        elif (platforms.PLATFORM == platforms.MAEMO5):
+            platforms.create_osso_context(values.OSSO_NAME, "1.0", False)
 
         # window
-        self.__window = Window(True)
-        self.__window.set_visible(True)
+        #self.__window = Window(True)
+        #self.__window.set_title("MediaBox")
+        #self.__window.set_size(800, 480)
+        #self.__window.set_size(480, 800)
+        #self.__window.set_visible(True)
 
-        if (maemo.IS_MAEMO):
-            self.__program.add_window(self.__window.get_gtk_window())
+        #if (platforms.PLATFORM == platforms.MAEMO4):
+        #    self.__program.add_window(self.__window.get_gtk_window())
 
-        self.__window.connect_key_pressed(self.__on_key)
-        self.__window.connect_key_released(lambda *a:self.__autorepeat_stop())
-        self.__window.connect_closed(self.__on_close_window)
+        #self.__window.connect_key_pressed(self.__on_key)
+        #self.__window.connect_key_released(lambda *a:self.__autorepeat_stop())
+        #self.__window.connect_closed(self.__on_close_window)
 
         # screen pixmap
         #screen = Pixmap(self.__window.window)
         
         Component.__init__(self)
         RootPane.__init__(self)
-        self.__window.add(self)
-        w, h = self.__window.get_size()
+        #self.__window.add(self)
+        #w, h = self.__window.get_size()
         
         #self.set_window(self.__window)
-        self.set_size(w, h)
+        #self.set_size(w, h)
         #self.set_screen(screen)
 
+        # viewer tabs
+        self.__tabs = Tabs()
+        self.add(self.__tabs)
+
+        # content box
         self.__box = UIWidget()
         self.add(self.__box)
         self.push_actor(self.__box)
-        
-
-        # image strip
-        self.__btn_strip = Button(">")
-        self.__btn_strip.set_visible(False)
-        self.__btn_strip.connect_clicked(self.__on_show_side_strip)
-        self.__box.add(self.__btn_strip)
-        
-        #self.__strip = ImageStrip(5)
-        #self.__strip.set_bg_color(theme.color_mb_background)
-        #self.__strip.set_visible(False)
-        #self.add(self.__strip)
-        
-        #self.__kscr = KineticScroller(self.__strip)
-        #self.__kscr.set_touch_area(0, 108)
-        #self.__kscr.add_observer(self.__on_observe_strip)
-
-        # title panel
-        self.__title_panel = TitlePanel()
-        self.__title_panel.set_title("Initializing")
-        self.__title_panel.set_visible(False)
-        self.__title_panel.connect_clicked(self.__on_click_title)
        
-        # control panel
-        self.__ctrl_panel = ControlPanel()
-        self.__ctrl_panel.set_visible(False)
-        self.__ctrl_panel.connect_button_pressed(self.__on_menu_button)
+        # title panel
+        #self.__title_panel = TitlePanel()
+        #self.__title_panel.set_title("Initializing")
+        #self.__title_panel.set_visible(False)
+        #self.__title_panel.connect_clicked(self.__on_click_title)
+        #self.__box.add(self.__title_panel)
+       
+        #self.__status_current_viewer = Image(None)
 
-        self.__status_current_viewer = Image(None)
-
-        gobject.timeout_add(0, self.__startup)
+        gobject.idle_add(self.__startup)
 
         
        
@@ -131,21 +123,21 @@ class AppWindow(Component, RootPane):
         Runs a queue of actions to take for startup.
         """
 
-        actions = [#(self.render_buffered, []),
+        actions = [#(self.render, []),
                    #(gtk.main_quit, []),
-                   (self.show_overlay, ["%s %s" % (values.NAME, values.VERSION),
-                                        "",
-                                        theme.mb_viewer_audio]),
-                   (self.__window.set_visible, [True]),
+                   #(self.show_overlay, ["%s %s" % (values.NAME, values.VERSION),
+                   #                     "",
+                   #                     theme.mb_viewer_audio]),
+                   #(self.__window.set_visible, [True]),
                    #(time.sleep, [5]),
-                   (self.__register_viewers, []),
+                   (self.__register_views, []),
+                   (self.__select_initial_view, []),
                    #(self.show_overlay, ["%s %s" % (values.NAME, values.VERSION),
                    #                     "- starting -",
                    #                     theme.mb_viewer_audio]),
-                   (self.__add_panels, []),
                    (self.__scan_at_startup, []),
-                   (self.hide_overlay, []),
-                   (self.__select_initial_viewer, []),
+                   #(self.hide_overlay, []),
+                   #(self.__add_panels, []),
                    (self.emit_message, [msgs.CORE_EV_APP_STARTED]),
                    ]
                    
@@ -187,36 +179,41 @@ class AppWindow(Component, RootPane):
         if (config.scan_at_startup()):
             self.__scan_media(True)
         else:
-            #self.emit_message(msgs.MEDIASCANNER_EV_SCANNING_FINISHED)
             self.__scan_media(False)
 
 
 
-    def __select_initial_viewer(self):
+    def __select_initial_view(self):
         """
-        Selects the viewer that is initially shown. Shows the menu if no
-        viewer was set.
+        Selects the view that is initially shown.
         """
         
-        self.__is_initialized = True
-        current_viewer = config.current_viewer()
-        current_device_id = config.current_device()
-        self.__select_viewer_by_name(current_viewer)
-        self.emit_message(msgs.UI_ACT_SELECT_VIEWER, current_viewer)
-        #self.emit_message(msgs.UI_ACT_SELECT_DEVICE, current_device_id)
+        self.__initial_view = self.__views[0]
+        self.__select_view_by_name(`self.__initial_view`)
 
 
-    def __register_viewers(self):
+    def __register_views(self):
         """
-        Sorts and registers the viewers.
+        Sorts and registers the views.
         """
 
-        self.__viewers.sort(lambda a,b:cmp(a.PRIORITY, b.PRIORITY))
+        self.__views.sort(lambda a,b:cmp(a.PRIORITY, b.PRIORITY))
 
-        for viewer in self.__viewers:
-            logging.info("registering viewer [%s]", viewer)
-            self.__box.add(viewer)
-            viewer.set_visible(False)
+        cnt = 0
+        for view in self.__views:
+            logging.info("registering view [%s]", view)
+            #self.add(view)
+            view.set_visible(False)
+            #view.set_title(`view`)
+            #self.__tabs.add_tab(None, view.TITLE, self.__on_show_view, view)
+            
+            if (cnt == 0):
+                view.connect_closed(self.__try_quit)
+            else:
+                view.connect_closed(self.__close_view)
+            
+            cnt += 1
+         #end for
 
 
     def __add_panels(self):
@@ -224,9 +221,9 @@ class AppWindow(Component, RootPane):
         Adds the panel components.
         """
     
-        self.__box.add(self.__title_panel)
-        self.__box.add(self.__ctrl_panel)
-        self.__title_panel.set_status_icon(self.__status_current_viewer)
+        #self.__box.add(self.__title_panel)
+        #self.__title_panel.set_status_icon(self.__status_current_viewer)
+        pass
 
 
     def render_this(self):
@@ -236,32 +233,26 @@ class AppWindow(Component, RootPane):
         w, h = self.get_size()
         screen = self.get_screen()
 
-        self.__title_panel.set_geometry(0, 0, w, 40)
-        self.__ctrl_panel.set_geometry(0, h - 70, w, 70)
-        #self.__strip.set_geometry(0, 0, 170, h)
-        self.__btn_strip.set_geometry(0, 40, 64, h - 110)
-
+        #self.__title_panel.set_geometry(0, 0, w, 40)
         
-        if (self.__view_mode == viewmodes.NORMAL):
-            if (self.__current_viewer):
-                self.__current_viewer.set_geometry(64, 40, w - 64, h - 110)
+        #if (self.__tabs.is_visible()):
+        if (w < h):
+            # portrait mode
+            #self.__tabs.set_orientation(Tabs.HORIZONTAL)
+            #self.__tabs.set_geometry(0, 0, w, 42)
+            if (self.__current_view):
+                self.__current_view.set_geometry(0, 0, w, h)
+        else:
+            # landscape mode
+            #self.__tabs.set_orientation(Tabs.VERTICAL)
+            #self.__tabs.set_geometry(0, 0, 42, h)
+            if (self.__current_view):
+                self.__current_view.set_geometry(0, 0, w, h)
 
-        elif (self.__view_mode == viewmodes.NO_STRIP):
-            if (self.__current_viewer):
-                self.__current_viewer.set_geometry(0, 40, w, h - 110)
+        #else:
+        #    if (self.__current_view):
+        #        self.__current_view.set_geometry(0, 0, w, h)
 
-        elif (self.__view_mode == viewmodes.NO_STRIP_PANEL):
-            if (self.__current_viewer):
-                self.__current_viewer.set_geometry(0, 0, w, h)
-            
-        elif (self.__view_mode == viewmodes.FULLSCREEN):
-            if (self.__current_viewer):
-                self.__current_viewer.set_geometry(0, 0, w, h)
-
-        elif (self.__view_mode == viewmodes.TITLE_ONLY):
-            if (self.__current_viewer):
-                self.__current_viewer.set_geometry(0, 40, w, h - 40)
-        
         
     def __set_view_mode(self, view_mode):
         """
@@ -274,49 +265,17 @@ class AppWindow(Component, RootPane):
         w, h = self.get_size()
 
         if (view_mode == viewmodes.NORMAL):
-            self.__title_panel.set_visible(True)
-            self.__ctrl_panel.set_visible(True)
-            self.__btn_strip.set_visible(True)
-            #if (self.__current_viewer):
-            #    self.__get_vstate().view_mode = view_mode
-                #self.__current_viewer.set_geometry(64, 40, w - 64, h - 110)
-                #self.__get_vstate().collection_visible = True
-            
-        elif (view_mode == viewmodes.NO_STRIP):
-            self.__title_panel.set_visible(True)
-            self.__ctrl_panel.set_visible(True)
-            self.__btn_strip.set_visible(False)
-            #if (self.__current_viewer):
-            #    self.__get_vstate().view_mode = view_mode
-                #self.__current_viewer.set_geometry(0, 40, w, h - 110)
-                #self.__get_vstate().collection_visible = False
-
-        #elif (view_mode == viewmodes.NO_STRIP_PANEL):
-        #    self.__title_panel.set_visible(True)
-        #    self.__ctrl_panel.set_visible(True)
-        #    self.__btn_strip.set_visible(False)
-        #    if (self.__current_viewer):            
-        #        self.__current_viewer.set_geometry(0, 0, w, h)
-        #        self.__get_vstate().collection_visible = False
-            
+            pass #self.__title_panel.set_visible(True)
+                       
         elif (view_mode == viewmodes.FULLSCREEN):
-            self.__title_panel.set_visible(False)
-            self.__ctrl_panel.set_visible(False)
-            self.__btn_strip.set_visible(False)
-            if (self.__current_viewer):
-                self.__current_viewer.set_geometry(0, 0, w, h)
-
-        #elif (view_mode == viewmodes.TITLE_ONLY):
-        #    self.__title_panel.set_visible(True)
-        #    self.__ctrl_panel.set_visible(False)
-        #    self.__btn_strip.set_visible(False)
-        #    if (self.__current_viewer):            
-        #        self.__current_viewer.set_geometry(0, 40, w, h - 40)
-        #        self.__get_vstate().collection_visible = False
+            #self.__title_panel.set_visible(False)
+            #if (self.__current_viewer):
+            #    self.__current_viewer.set_geometry(0, 0, w, h)
+            pass
 
         self.__view_mode = view_mode
-        if (self.__current_viewer):
-            self.__get_vstate().view_mode = view_mode
+        #if (self.__current_viewer):
+        #    self.__get_vstate().view_mode = view_mode
 
 
     def __on_click_title(self):
@@ -361,19 +320,24 @@ class AppWindow(Component, RootPane):
         self.emit_message(msgs.MEDIASCANNER_ACT_SCAN, mediaroots, force_rebuild)
         
 
-        for v in self.__viewers:
-            vstate = self.__get_vstate(v)
-            vstate.selected_item = -1
-            vstate.item_offset = 0
+        #for v in self.__viewers:
+        #    vstate = self.__get_vstate(v)
+        #    vstate.selected_item = -1
+        #    vstate.item_offset = 0
 
 
-        import gc; gc.collect()
+        #import gc; gc.collect()
         #self.hide_overlay()
 
 
-    def __on_show_side_strip(self):
-        
-        self.emit_message(msgs.UI_ACT_SHOW_STRIP)
+    def __on_show_view(self, view):
+    
+        if (self.__current_view):
+            self.__current_view.set_visible(False)
+    
+        self.__current_view = view
+        self.__current_view.set_visible(True)
+        self.render()
 
                         
     def __on_close_window(self):
@@ -484,7 +448,7 @@ class AppWindow(Component, RootPane):
         Displays the virtual keyboard.
         """
        
-        self.emit_message(msgs.VKB_ACT_SHOW, self.__window)
+        pass #self.emit_message(msgs.VKB_ACT_SHOW, self.__window)
 
 
 
@@ -540,34 +504,44 @@ class AppWindow(Component, RootPane):
         return self.__viewer_states[viewer]
                 
             
- 
-    def __select_viewer_by_name(self, name):
+    def __select_view_by_name(self, name):
         """
-        Selects the current viewer by name.
+        Selects the current view by name.
         """
         
-        viewers = [ v for v in self.__viewers if repr(v) == name ]
-        if (viewers):
-            idx = self.__viewers.index(viewers[0])
-            self.__select_viewer(idx)
-        else:
-            self.__select_viewer(0)
+        views = [ v for v in self.__views if repr(v) == name ]
+        if (views):
+            view = views[0]
             
+            if (view == self.__current_view):
+                return
+            
+            if (self.__current_view and self.__current_view != self.__initial_view):
+                self.__current_view.set_visible(False)
+    
+            self.__current_view = view
+            self.__current_view.set_visible(True)
+            self.__current_view.render()
+        #end if
+
+
+    def __close_view(self):
+    
+        self.__select_view_by_name(`self.__initial_view`)
 
             
+    """
     def __select_viewer(self, idx):
-        """
+        ""
         Selects the current viewer by index number.
-        """        
+        ""        
       
         viewer = self.__viewers[idx]
         
         if (self.__current_viewer):
             self.__current_viewer.hide()
-            #self.__get_vstate().item_offset = self.__strip.get_offset()
                 
-        self.__current_viewer = viewer
-        #self.__kscr.stop_scrolling()
+        self.__current_view = viewer
 
         vstate = self.__get_vstate()
         self.__set_view_mode(vstate.view_mode)
@@ -575,32 +549,39 @@ class AppWindow(Component, RootPane):
         self.set_frozen(True)
         viewer.show()
         
-        #self.__strip.set_offset(vstate.item_offset)
-        #if (vstate.selected_item >= 0):
-        #    self.__hilight_item(vstate.selected_item)
-
         icon = viewer.ICON.scale_simple(32, 32, gtk.gdk.INTERP_TILES)
         self.__status_current_viewer.set_image(icon)
         self.__title_panel.set_status_icon(self.__status_current_viewer)
         
         self.set_frozen(False)
         self.fx_slide_in()
-        #self.render_buffered()
         
         self.emit_message(msgs.INPUT_ACT_REPORT_CONTEXT)
         self.emit_message(msgs.UI_EV_VIEWER_CHANGED, idx)
-      
+    """
+
+    """
+    def __stack_window(self):
+    
+        self.__window.remove(self)
+        win = Window(True)
+        win.add(self)
+        win.set_visible(True)
+        self.set_visible(True)
+        win.render()
+    """
         
     def __try_quit(self):
     
-        self.__window.present()
-        result = self.call_service(msgs.DIALOG_SVC_QUESTION,
-                                   "Exit",
-                                   "Do you want to quit MediaBox?")
-
+        #self.__window.present()
+        #result = self.call_service(msgs.DIALOG_SVC_QUESTION,
+        #                           "Exit",
+        #                           "Do you want to quit?")
+        result = 0
+        
         if (result == 0):
-            config.set_current_viewer(self.__current_viewer)
-            config.set_current_device(self.__current_device_id)
+            #config.set_current_viewer(self.__current_viewer)
+            #config.set_current_device(self.__current_device_id)
             self.emit_message(msgs.MEDIA_ACT_STOP)
             self.emit_message(msgs.CORE_EV_APP_SHUTDOWN)
             gtk.main_quit()
@@ -610,15 +591,16 @@ class AppWindow(Component, RootPane):
 
     def handle_COM_EV_COMPONENT_LOADED(self, component):
     
-        if (isinstance(component, Viewer)):
-            self.__viewers.append(component)
+        if (isinstance(component, View)):
+            self.__views.append(component)
+            
         elif (isinstance(component, Widget)):
             self.add(component)
 
 
     def handle_CORE_ACT_APP_MINIMIZE(self):
 
-        self.__window.iconify()
+        pass #self.__window.iconify()
         
         
     def handle_CORE_ACT_APP_CLOSE(self):
@@ -645,18 +627,13 @@ class AppWindow(Component, RootPane):
 
     def handle_CORE_ACT_SET_TITLE(self, title):
 
-        self.__title_panel.set_title(title)
-        self.__window.set_title(title)
+        #self.__title_panel.set_title(title)
+        pass #self.__window.set_title(title)
 
 
     def handle_CORE_ACT_SET_INFO(self, info):
 
-        self.__title_panel.set_info(info)
-
-
-    def handle_CORE_ACT_SET_TOOLBAR(self, tbset):
-
-        self.__ctrl_panel.set_toolbar(tbset)
+        pass #self.__title_panel.set_info(info)
 
 
     def handle_HWKEY_EV_BACKSPACE(self):
@@ -672,7 +649,7 @@ class AppWindow(Component, RootPane):
         term += key.lower()
         self.__set_search_term(term)
 
-
+    """
     def handle_INPUT_EV_PREVIOUS_VIEWER(self):            
 
         idx = self.__viewers.index(self.__current_viewer)
@@ -684,22 +661,23 @@ class AppWindow(Component, RootPane):
         idx = self.__viewers.index(self.__current_viewer)
         if (idx < len(self.__viewers) - 1):
             self.__select_viewer(idx + 1)
+    """
 
 
     def handle_MEDIASCANNER_EV_SCANNING_STARTED(self):
 
-        self.show_overlay("Scanning Media", "", theme.mb_viewer_audio)
+        pass #self.show_overlay("Scanning Media", "", theme.mb_viewer_audio)
         
         
     def handle_MEDIASCANNER_EV_SCANNING_PROGRESS(self, name):
 
-        self.show_overlay("Scanning Media", "- %s -" % name,
-                          theme.mb_viewer_audio)
+        pass #self.show_overlay("Scanning Media", "- %s -" % name,
+        #                  theme.mb_viewer_audio)
 
     
     def handle_MEDIASCANNER_EV_SCANNING_FINISHED(self):
 
-        self.hide_overlay()
+        pass #self.hide_overlay()
 
 
     def handle_SYSTEM_EV_BATTERY_REMAINING(self, percent):                
@@ -731,17 +709,17 @@ class AppWindow(Component, RootPane):
 
     def handle_UI_ACT_SHOW_MESSAGE(self, text, subtext, icon):
 
-        self.show_overlay(text, subtext, icon)
+        pass #self.show_overlay(text, subtext, icon)
 
 
     def handle_UI_ACT_HIDE_MESSAGE(self):
 
-        self.hide_overlay()
+        pass #self.hide_overlay()
 
 
-    def handle_UI_ACT_SELECT_VIEWER(self, name):
+    def handle_UI_ACT_SELECT_VIEW(self, name):
 
-        self.__select_viewer_by_name(name)
+        self.__select_view_by_name(name)
 
 
     def handle_UI_ACT_VIEW_MODE(self, mode):
@@ -751,10 +729,22 @@ class AppWindow(Component, RootPane):
 
     def handle_UI_ACT_SET_STATUS_ICON(self, w):
     
-        self.__title_panel.set_status_icon(w)
+        pass #self.__title_panel.set_status_icon(w)
         
 
     def handle_UI_ACT_UNSET_STATUS_ICON(self, w):
     
-        self.__title_panel.unset_status_icon(w)
+        pass #self.__title_panel.unset_status_icon(w)
+
+
+    def handle_ASR_EV_LANDSCAPE(self):
+    
+        for view in self.__views:
+            view.set_portrait_mode(False)
+        
+        
+    def handle_ASR_EV_PORTRAIT(self):
+
+        for view in self.__views:    
+            view.set_portrait_mode(True)
 
