@@ -2,17 +2,27 @@ from Widget import Widget
 from ui import try_rgba
 from Pixmap import Pixmap
 from Widget import Widget
-from utils import maemo
+import platforms
 
 import gtk
+import os
 
 
 class Window(Widget):
+    """
+    Base class for windows.
+    """
 
     EVENT_CLOSED = "event-closed"
     
 
-    def __init__(self, is_toplevel):
+    def __init__(self, is_toplevel, hide_cursor = False):
+        """
+        Creates a new window.
+        
+        @param is_toplevel: whether the window is a toplevel window
+        @param hide_cursor: whether to hide the mouse pointer cursor
+        """
     
         self.__size = (0, 0)
         self.__fixed = None
@@ -20,19 +30,22 @@ class Window(Widget):
         Widget.__init__(self)
     
         if (is_toplevel):
-            if (maemo.IS_MAEMO):
+            if (platforms.PLATFORM in (platforms.MAEMO5, platforms.MER)):
+                import hildon
+                self.__window = hildon.StackableWindow()
+
+            elif (platforms.PLATFORM == platforms.MAEMO4):
                 import hildon
                 self.__window = hildon.Window()
                 self.__window.fullscreen()
+                
             else:
                 self.__window = gtk.Window(gtk.WINDOW_TOPLEVEL)
+                self.__window.set_default_size(800, 480)
             
-            #self.set_size(800, 480)
-            #self.__window.set_resizable(False)
         else:
             self.__window = gtk.Window(gtk.WINDOW_POPUP)
 
-
         self.__window.connect("configure-event", self.__on_configure)
         self.__window.connect("expose-event", self.__on_expose)
         self.__window.connect("button-press-event", self.__on_button_pressed)
@@ -51,23 +64,44 @@ class Window(Widget):
                                  gtk.gdk.KEY_RELEASE_MASK)
 
 
-        self.__window.set_size_request(800, 480)
+        self.set_visible(False)
         try_rgba(self.__window)
         self.__window.set_app_paintable(True)
         self.__window.realize()
 
 
+        if (platforms.PLATFORM in (platforms.MAEMO5, platforms.MER)):
+            import hildon
+            # we need to notify Maemo5 that we want to use the volume keys
+            self.__window.window.property_change("_HILDON_ZOOM_KEY_ATOM",
+                                                 "XA_INTEGER", 32,
+                                                 gtk.gdk.PROP_MODE_REPLACE,
+                                                 [1])                
+
+
         self.__fixed = gtk.Fixed()
         self.__fixed.show()
         self.__window.add(self.__fixed)
-        self.__screen = None #Pixmap(self.__window.window)
+        self.__screen = None
 
+        # hide mouse cursor
+        if (hide_cursor):
+            pbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 8, 8)
+            pbuf.fill(0x00000000)
+            csr = gtk.gdk.Cursor(gtk.gdk.display_get_default(), pbuf, 0, 0)
+            self.__window.window.set_cursor(csr)
+            del pbuf
+        #end if
                 
         self.set_window(self)
-        self.__check_window_size()
 
 
     def get_gtk_window(self):
+        """
+        Returns the associated GtkWindow of this window.
+        
+        @return: GtkWindow object
+        """
     
         return self.__window
 
@@ -94,12 +128,13 @@ class Window(Widget):
 
     def __on_expose(self, src, ev):
     
-        x, y, w, h = ev.area
-        self.__screen.restore(x, y, w, h)
+        if (self.__screen):
+            x, y, w, h = ev.area
+            self.__screen.restore(x, y, w, h)
 
 
     def __on_close_window(self, src, ev):
-    
+        
         self.send_event(self.EVENT_CLOSED)
         return True
         
@@ -183,7 +218,8 @@ class Window(Widget):
     def set_size(self, w, h):
     
         Widget.set_size(self, w, h)
-        self.__window.set_size_request(w, h)
+        #self.__window.set_size_request(w, h)
+        self.__window.resize(w, h)
 
 
     def get_size(self):
@@ -221,6 +257,11 @@ class Window(Widget):
     def present(self):
     
         self.__window.present()
+        
+        
+    def fullscreen(self):
+    
+        self.__window.fullscreen()
 
 
     def iconify(self):
@@ -232,3 +273,56 @@ class Window(Widget):
     
         self.__window.set_title(title)
 
+
+    def __set_portrait_property(self, prop, value):
+
+        self.__window.window.property_change(prop, "CARDINAL", 32,
+                                             gtk.gdk.PROP_MODE_REPLACE,
+                                             [value])
+    
+    
+    def set_portrait_mode(self, v):
+        """
+        Switches between landscape and portrait mode on supported platforms.
+        @since: 2009.09
+        """
+        
+        if (platforms.PLATFORM == platforms.MAEMO5):
+            if (v):
+                self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
+                self.__set_portrait_property("_HILDON_PORTRAIT_MODE_REQUEST", 1)
+            else:
+                self.__set_portrait_property("_HILDON_PORTRAIT_MODE_SUPPORT", 1)
+                self.__set_portrait_property("_HILDON_PORTRAIT_MODE_REQUEST", 0)
+
+        elif (platforms.PLATFORM == platforms.MAEMO4):
+            if (v):
+                os.system("xrandr -o left")
+            else:
+                os.system("xrandr -o normal")
+
+        #end if
+
+
+    def set_window_menu(self, *items):
+        """
+        Sets the window menu.
+        @since: 2009.10
+        """
+        
+        if (platforms.PLATFORM == platforms.MAEMO5):
+            import hildon
+            menu = hildon.AppMenu()
+        
+            for item in items:
+                icon, label, callback = item
+
+                btn = gtk.Button(label)
+                btn.connect("clicked", lambda x, cb:cb(), callback)
+                menu.append(btn)
+            #end for
+
+            menu.show_all()            
+            self.__window.set_app_menu(menu)
+        #end if
+
